/* ************************************************************************** 
 *             The Pebbles Personal Digital Assistant Project               *
 * **************************************************************************
 * This code was written as part of the Pebbles project at                  *
 * Carnegie Mellon University.  See the following for more information:     *
 *                http://www.cs.cmu.edu/~pebbles                            *
 * This code is being made available for free general non-commercial use.   *
 * For other conditions, see http://www.cs.cmu.edu/~pebbles/conditions.html *
 * Please contact us for more information: pebbles@cs.cmu.edu               *  
 *         Copyright (C) 1998 - Carnegie Mellon University                  *
 * **************************************************************************/

/* 
   This file contains the PC plugin for Ping.
   Ping is a sample Pebbles program that triggers a beep on the PC
   when the user presses a button on the PDA.

   Designed and implemented by Rob Miller
*/


#include "stdafx.h"
#include <afxdllx.h>
#include "Ping.h"
#include "..\PingMessages.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


static AFX_EXTENSION_MODULE PingDLL = { NULL, NULL };

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	// Remove this if you use lpReserved
	UNREFERENCED_PARAMETER(lpReserved);

	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("Ping.DLL Initializing!\n");
		
		// Extension DLL one-time initialization
		if (!AfxInitExtensionModule(PingDLL, hInstance))
			return 0;

		// Insert this DLL into the resource chain
		// NOTE: If this Extension DLL is being implicitly linked to by
		//  an MFC Regular DLL (such as an ActiveX Control)
		//  instead of an MFC application, then you will want to
		//  remove this line from DllMain and put it in a separate
		//  function exported from this Extension DLL.  The Regular DLL
		//  that uses this Extension DLL should then explicitly call that
		//  function to initialize this Extension DLL.  Otherwise,
		//  the CDynLinkLibrary object will not be attached to the
		//  Regular DLL's resource chain, and serious problems will
		//  result.

		new CDynLinkLibrary(PingDLL);
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("Ping.DLL Terminating!\n");
		// Terminate the library before destructors are called
		AfxTermExtensionModule(PingDLL);
	}
	return 1;   // ok
}

extern "C" BOOL WINAPI PebblesMain (HWND hwnd, PebblesPlugin *pPlugin)
{
    CPing *thread = new CPing();
    thread->CreateThread();

    pPlugin->thread = thread->m_nThreadID;
    pPlugin->hasAbout = true;
    pPlugin->hasPrefs = false;
	pPlugin->hThread = thread->m_hThread;
    return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CPing

IMPLEMENT_DYNCREATE(CPing, CWinThread)

CPing::CPing()
{
}

BOOL CPing::InitInstance()
{
	// Initialize plugin here

    return TRUE;
}

int CPing::ExitInstance()
{
	// Clean up resources here

    return CWinThread::ExitInstance();
}

BEGIN_MESSAGE_MAP(CPing, CWinThread)
	//{{AFX_MSG_MAP(CPing)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_PEBBLES_RECEIVED,    OnReceived)
	ON_MESSAGE(WM_PEBBLES_NEW_USER,    OnNewUser)
	ON_MESSAGE(WM_PEBBLES_DONE_USER,   OnDoneUser)
	ON_MESSAGE(WM_PEBBLES_ABOUT,       OnAbout)
	ON_MESSAGE(WM_PEBBLES_PREFS,       OnPrefs)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPing message handlers

LRESULT CPing::OnReceived(WPARAM wParam, LPARAM lParam)
{
    PebblesUser *user = (PebblesUser *)wParam;
    char *message = (char *)lParam;

    // Dissect the message
    char cmd = PEBBLES_CMD (message);
    int len = PEBBLES_LEN (message);
    char *data = PEBBLES_DATA (message);

	HANDLE h;

    switch (cmd) {
    case CMD_PING:
        // Respond to ping by beeping
        ::MessageBeep (MB_OK);

        // Then send a pong back
        {
            // Construct the CMD_PONG reply, passing back the same data
            // that was passed with the CMD_PING message
			h = GlobalAlloc(0, PEBBLES_HEADER_SIZE + len);
            char *reply = (char *) GlobalLock(h);
            PEBBLES_SET_CMD(reply, CMD_PONG);
            PEBBLES_SET_LEN(reply, len);
            memcpy (PEBBLES_DATA(reply), data, len);

            // Send the CMD_PONG back to the handheld app
            ::PostThreadMessage (user->thread, WM_PEBBLES_SEND, (int)user, (int)reply);
        }
        break;
    }

    // free the message
	h = GlobalHandle(message);
    GlobalFree(h);
    return 1L;
}

LRESULT CPing::OnNewUser(WPARAM wParam, LPARAM lParam)
{
    PebblesUser *user = (PebblesUser *)wParam;

    return 1L;
}

LRESULT CPing::OnDoneUser(WPARAM wParam, LPARAM lParam)
{
    PebblesUser *user = (PebblesUser *)wParam;

    return 1L;
}

LRESULT CPing::OnAbout(WPARAM wParam, LPARAM lParam)
{
    HWND hwndPebblesPC = (HWND) wParam;

    CDialog dlg ("ABOUT_PING");
    dlg.DoModal ();
    return 1L;
}

LRESULT CPing::OnPrefs(WPARAM wParam, LPARAM lParam)
{
    HWND hwndPebblesPC = (HWND) wParam;

    return 1L;
}
