/* ************************************************************************** 
 *             The Pebbles Personal Digital Assistant Project               *
 * **************************************************************************
 * This code was written as part of the Pebbles project at                  *
 * Carnegie Mellon University.  See the following for more information:     *
 *                http://www.cs.cmu.edu/~pebbles                            *
 * This library code is available for use by others.                        *
 * Please contact us for more information: pebbles@cs.cmu.edu               *  
 *                                                                          *
 *         Copyright (C) 1998-2000 - Carnegie Mellon University             *
 * **************************************************************************/


/* 
   This file contains <CONTENTS>
   Designed and implemented by Rob Miller
*/

//
// Pebbles library for PalmPilot.
//
// This file defines a set of functions that implement the app-side
// of the Pebbles protocol, for PalmPilot apps.
//

#ifndef _PEBBLES_PILOT_H_
#define _PEBBLES_PILOT_H_

#include <DLServer.h> // for dlkMaxUserNameLength
#include "NdkLib.h"

#define MAX_ULONG 0xFFFFFFFF;

// System version numbers for FtrGet
#define version20   0x02000000
#define version30   0x03000000	
#define version35	0x03500000

//
// Typedefs for Palm 3.3 compatibility
//

typedef Int16 Int;
typedef Int32 Long;

typedef UInt16 Word;
typedef UInt16 UInt;
typedef UInt32 DWord;
typedef UInt32 ULong;

typedef void* VoidPtr;
typedef char* CharPtr;
typedef MemPtr Ptr;
typedef UInt16* UIntPtr;
typedef Int16* IntPtr;


typedef MemHandle Handle;
typedef MemHandle VoidHand;


//
// Connection state.
//
typedef enum {
    SERIAL   = 0,
    INFRARED = 1,
	NETWORK   = 2
} PebblesConnectionType;


typedef struct _PebblesConnection {

	char *	  plugin;		// Name of plugin to use on desktop (null-terminated)

	UInt16	  ref;			// Handle of open serial port library
	
	PebblesConnectionType connType; // Serial or infrared

	
	Boolean	  connected;	// True iff connected to an active plugin on
							// the desktop (i.e., a CMD_ACK has been
							// received from the desktop)
							
	Boolean   pluginConnectError;   	// True if a connection to the PC was made,
									// but no plugin .DLL was found
									
	Err		  lastErr;		// last system error encountered by Pebbles functions

	int 		  sound_on_connected; // sound to make on connection, 0 = no sound, default = sndConfirmation
	
    void (*connectionCallback) (struct _PebblesConnection *);
                            // Callback when connection is made; NULL for none

    UInt32 repeatDeadline;   // Tick count (see Palm API function TimGetTicks)
                            // at which to repeat the CHANGE_PLUGIN message
     
     char userName[dlkMaxUserNameLength + 1];
     int userNameLength;

	char * receiveBuffer;  
	
	SocketType *socket;

} PebblesConnection;

typedef PebblesConnection *PebblesPtr;

//
// API functions.
//

Boolean PebblesOpen (PebblesPtr pbl, char *plugin);
Boolean PebblesOpenAndWait (PebblesPtr pbl, char *plugin);
Boolean PebblesOpenAndWaitSetSound (PebblesPtr pbl, char *plugin, int sound);
void PebblesSetConnectionSound(PebblesPtr pbl,  int sound);

void PebblesClose (PebblesPtr pbl);

Boolean PebblesChangePlugin (PebblesPtr pbl, char *plugin);
void PebblesSetConnectionCallback (PebblesPtr pbl, void (*callback) (PebblesPtr pbl));
Boolean PebblesHandshake (PebblesPtr pbl, int timeout);

Boolean PebblesSend (PebblesPtr pbl, unsigned char cmd, void *data, unsigned len);
Boolean PebblesSendHeader (PebblesPtr pbl, unsigned char cmd, unsigned len);
Boolean PebblesSendData (PebblesPtr pbl, void *data, unsigned len);

unsigned char PebblesReceiveHeader (PebblesPtr pbl, unsigned *pLen, int timeout);
Boolean PebblesReceiveData (PebblesPtr pbl, void *data, unsigned len, int timeout);

void PebblesReceiveFlush (PebblesPtr pbl);

Boolean PebblesConnectionFormHandleEvent(PebblesPtr pbl, EventPtr event);

// copies user name into nameptr which should be of size dlkMaxUserNameLength + 1
void PebblesGetUserName (char *nameptr);

void PebblesDisableAutoPowerOff();
void PebblesEnableAutoPowerOff();
#endif
