//
// Pebbles library for WinCE.
//
// This file defines a set of functions that implement the app-side
// of the Pebbles protocol, for WinCE apps.
//

#ifndef _PEBBLES_CE_H_
#define _PEBBLES_CE_H_

#include <windows.h>

//
// Connection state.
//

// Device types.
typedef enum {
    SERIAL   = 0,
    INFRARED = 1,
	NETWORK   = 2
} PebblesConnectionType;


// Description of a Pebbles connection
typedef struct _PebblesConnection {

	char *	  plugin;		// Name of plugin to use on desktop (null-terminated)


        PebblesConnectionType connType; // Network, Serial or infrared

        HANDLE	  hPort;		// Handle of open port library
	
	BOOL	  connected;            // True iff connected to an active plugin on
                                        // the desktop (i.e., a CMD_ACK has been
                                        // received from the desktop)
	char *ipAddr;				// IP Address of remote computer

	BOOL	pluginConnectError;   	// True if a connection to the PC was made,
									// but no plugin .DLL was found

	HANDLE    hWatchdogThread;	// Thread watching for power-cycle

        void (*connectionCallback) (struct _PebblesConnection *);
                                        // Callback when connection is made

        DWORD     repeatDeadline;       // Tick count (see WinCE API function GetTickCount)
                                        // at which to repeat the CHANGE_PLUGIN message
} PebblesConnection;

typedef PebblesConnection *PebblesPtr;



//
// API functions.
//

BOOL PebblesOpen (PebblesPtr pbl, char *plugin, PebblesConnectionType connType = SERIAL, char *networkAddr=NULL);
void PebblesClose (PebblesPtr pbl);

BOOL PebblesChangePlugin (PebblesPtr pbl, char *plugin);
BOOL PebblesHandshake (PebblesPtr pbl);

BOOL PebblesSend (PebblesPtr pbl, char cmd, void *data, unsigned len);
BOOL PebblesSendHeader (PebblesPtr pbl, char cmd, unsigned len);
BOOL PebblesSendData (PebblesPtr pbl, void *data, unsigned len);

void PebblesSetConnectionCallback (PebblesPtr pbl, void (*callback) (PebblesPtr pbl));

char PebblesReceiveHeader (PebblesPtr pbl, unsigned int *pLen, int timeout);
BOOL PebblesReceiveData (PebblesPtr pbl, void *data, unsigned int len, int timeout, void (*callback) (int percentComplete) = NULL);

void PebblesReceiveFlush (PebblesPtr pbl);

#endif
