/* ************************************************************************** 
 *             The Pebbles Personal Digital Assistant Project               *
 * **************************************************************************
 * This code was written as part of the Pebbles project at                  *
 * Carnegie Mellon University.  See the following for more information:     *
 *                http://www.cs.cmu.edu/~pebbles                            *
 * This code is being made available for free general non-commercial use.   *
 * For other conditions, see http://www.cs.cmu.edu/~pebbles/conditions.html *
 * Please contact us for more information: pebbles@cs.cmu.edu               *  
 *         Copyright (C) 1999 - Carnegie Mellon University                  *
 * **************************************************************************/

/* 
   This file contains the WindowsCE port of the Ping application.
   Designed and implemented by Franklin Chen.
   Based on Palm Ping.c.
*/

// Ping.cpp : Defines the entry point for the application.
//


#include <windows.h>
#include <commctrl.h>
#include <Pebbles.h>
#include "PebblesCE.h"
#include "resource.h"
#include "..\PingMessages.h"


#define HOSTNAME_LEN 20

/*** Global variables. ***/
PebblesConnection pbl;

PebblesConnectionType connType = SERIAL;

char szHostname[HOSTNAME_LEN];

TCHAR pszAppName[] = TEXT("Ping");
TCHAR pszTitle[] = TEXT("Ping");

HINSTANCE ghInst;

/* Define the HWNDs used by this application
   hwndMain       -->     Main application window
   hwndCB         -->     Command bar
   hwndButton     -->     HWND of ping button
 */
HWND hwndMain;
HWND hwndCB;
HWND hwndButton;


// name of Pebbles plugin DLL corresponding to this app
#define PLUGIN_NAME		"Ping"

// interbyte timeout for receiving messages, in timer ticks (1/100 sec)
#define TIMEOUT			10

#define ID_TIMER	107

/*** Function declarations. ***/

/* Define the main application window procedure */
LRESULT CALLBACK WndProc(HWND hwnd,
						 UINT message,
						 WPARAM wParam,
						 LPARAM lParam);



/**************************************************************************************

   Preferences

 **************************************************************************************/


void SaveOptionsToRegistry() {
    DWORD err, createFlag;
    HKEY hNew = NULL;

    // Save all options in registry
    err = RegCreateKeyEx(HKEY_CURRENT_USER, 
			L"Software\\Pebbles\\Ping", 
			0,
			NULL,
			0, 0, NULL,
			&hNew, &createFlag);
    if(err == ERROR_SUCCESS) {
	 RegSetValueEx(hNew, L"ConnType", 0, REG_BINARY, (unsigned char *) &connType, sizeof(connType));
	 RegSetValueEx(hNew, L"Hostname", 0, REG_SZ, (unsigned char *) &szHostname, strlen(szHostname)+1);

	 RegCloseKey(hNew);
    } 
}


void LoadOptionsFromRegistry() {
    DWORD err, queryType, querySize;
    HKEY hNew = NULL;

    err = RegOpenKeyEx(HKEY_CURRENT_USER, 
			L"Software\\Pebbles\\Ping", 
			0,
			KEY_READ,
			&hNew);
    if(err == ERROR_SUCCESS) {
     querySize = 1;
     RegQueryValueEx(hNew, L"ConnType", 0, &queryType, (unsigned char *) &connType, &querySize);
	 
	 querySize = HOSTNAME_LEN;
	 RegQueryValueEx(hNew, L"Hostname", 0, &queryType, (unsigned char *) &szHostname, &querySize);

     RegCloseKey(hNew);
    }
}


/****************************************************************************

   PCConnect - Connect to the PC using IR, Serial Port, or through a Socket

   Globals:  PLUGIN_NAME, connType, szHostname

   Notes:  generic.

  ****************************************************************************/

void PCConnect ()
{
    if(!PebblesOpen(&pbl, PLUGIN_NAME, connType, szHostname)) {
		switch(connType) {
		case SERIAL:
			MessageBox(NULL, L"Serial Port not available.", TEXT("Error"), MB_OK);
			break;
		case INFRARED:
			MessageBox(NULL, L"Infrared Port not available.", TEXT("Error"), MB_OK);
			break;
		case NETWORK:
			//MessageBox(NULL, L"Network not available.", TEXT("Error"), MB_OK);
			break;
		}
    }

}

/****************************************************************************

   PCDisconnect - Disconnect from the PC

   Globals:  pbl

   Notes:  generic.

  ****************************************************************************/

void PCDisconnect () {
    PebblesClose(&pbl);
}


/****************************************************************************

  ReceiveCheck - Handle messages sent from Pebbles

  Similar to Palm.

  Globals:  pbl, CMD_PONG

  ****************************************************************************/

void ReceiveCheck ()
{
  unsigned len;

	switch (PebblesReceiveHeader (&pbl, &len, 0)) {
	case CMD_PONG:
		// received a Pong message -- beep in response
	  MessageBeep(0);
		
		// Read the message's data.
		// We don't care about the data, so pass a NULL pointer to throw
		// away the received bytes.
		PebblesReceiveData (&pbl, NULL, len, TIMEOUT);
		break;
	}
}

/****************************************************************************

   CreateCommandBar - creates the menu bar and the application-specific
   toolbars.  The handle of the main window is passed
   as a parameter.

   Globals:  ghInst, hwndCB, IDR_OPTIONSMENU

  ****************************************************************************/

void CreateCommandBar(HWND hwnd) {
  hwndCB = CommandBar_Create(ghInst, hwnd, 1);

  // Options menu
  CommandBar_InsertMenubar(hwndCB, ghInst, IDR_OPTIONSMENU, 0);

  // add the close button
  CommandBar_AddAdornments(hwndCB, 0, 0);

}

/****************************************************************************

   AboutDialogProc - Handle Events in the About Dialog (close event)

   Notes:  generic

  ****************************************************************************/

BOOL CALLBACK AboutDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// just close the dialog if a command is sent
	if(uMsg == WM_COMMAND) {
		EndDialog(hwndDlg, 0);
		return (TRUE);
	} else if(uMsg == WM_INITDIALOG) {
		return (TRUE);
	}
	
	return FALSE;
}


/****************************************************************************

   OptionsDialogProc - Handle Events in the Options Dialog

   Notes:  generic

  ****************************************************************************/

BOOL CALLBACK OptionsDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	TCHAR szWHostname[20];
		
	switch(uMsg) {
	case WM_INITDIALOG:
		{
		  mbstowcs(szWHostname, szHostname,HOSTNAME_LEN);

		  SetDlgItemText(hwndDlg, IDC_HOSTNAME, szWHostname);
          SendMessage (GetDlgItem (hwndDlg, IDC_SERIAL+connType), BM_SETCHECK, TRUE, 0);
        }
		return(TRUE);	
	break;
	case WM_COMMAND:
		if(wParam == IDCANCEL) {
			EndDialog(hwndDlg, FALSE);
			return(TRUE);
		}
		if(wParam == IDOK) {
			// figure out which item was selected
			for(int i=0; i<3; i++) {
				if(SendMessage(GetDlgItem (hwndDlg, IDC_SERIAL+i), BM_GETCHECK, 0, 0)) 
					connType = (PebblesConnectionType) i;
			}
			if(connType == NETWORK) {
				if(!GetDlgItemText(hwndDlg, IDC_HOSTNAME, szWHostname, HOSTNAME_LEN)) {
					MessageBox(NULL, L"A Hostname must be entered.", L"Error", MB_ICONEXCLAMATION | MB_OK);
					return (TRUE);
				}
				wcstombs(szHostname, szWHostname, HOSTNAME_LEN);
			}
			EndDialog(hwndDlg, TRUE);
			return (TRUE);
		}
	break;
	default:
		return FALSE;
	}

	return FALSE;
}	



/* Globals: pbl, CMD_PING. */
LRESULT CALLBACK WndProc(HWND hwnd,
						 UINT message,
						 WPARAM wParam,
						 LPARAM lParam)
{
  switch (message) {
  case WM_CREATE:
    CreateCommandBar(hwnd);

    return 0;

  case WM_COMMAND:
    {
      UINT nID = LOWORD(wParam);

      switch (nID) {
      case ID_OPTIONS_ABOUT:
	(void) DialogBox(ghInst, MAKEINTRESOURCE(ID_ABOUTDLG),
			 hwndMain,
			 AboutDialogProc);
	break;

      case ID_OPTIONS_CONNECTIONOPTIONS:
	// Generic
	// if the user presses OK on the options dialog, disconnect and reconnect
	if (DialogBox(ghInst, MAKEINTRESOURCE(IDD_OPTIONSDLG),
		      hwndMain, OptionsDlgProc)) {
	  PCDisconnect();
	  PCConnect();
	}
	break;

      case IDC_BUTTON:
	PebblesSend(&pbl, CMD_PING, "hello", 5);

	break;
      default:
	break;
      }

      return 0;
    }

  case WM_TIMER:
    ReceiveCheck();
    return 0;

  case WM_CLOSE:
    PCDisconnect();
    DestroyWindow(hwnd);
    return 0;


  case WM_DESTROY:
    CommandBar_Destroy(hwndCB);
    PostQuitMessage(0);
    return 0;


  default:
    return DefWindowProc(hwnd, message, wParam, lParam);
  }
}




/* Globals: ghInst, hwndMain, hwndButton
   pszAppName, pszTitle
 */
BOOL InitInstance (HINSTANCE hInstance, int nCmdShow )
{
	/* Save application instance in ghInst for
	   possible use by other functions, such as
	   the main windows's window procedure.
	 */
	ghInst = hInstance;
	hwndMain = CreateWindow(pszAppName,
                            pszTitle,
				WS_VISIBLE,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
							NULL,
							NULL,
							hInstance,
							NULL);
	if (!hwndMain) {
	  return FALSE;
	}

	/* Create the ping button. */
	hwndButton = CreateWindow(TEXT("BUTTON"),
				  TEXT("Ping"),
				  WS_VISIBLE|WS_CHILD,
				  50,50,100,100,
				  hwndMain,
				  (HMENU)IDC_BUTTON,
				  hInstance,
				  NULL);
	if (!hwndButton) {
	  return FALSE;
	}

	PCConnect();

	// call PebblesHandshake() every 1/2 second
	SetTimer(hwndMain, ID_TIMER, 500, NULL);

	ShowWindow(hwndMain, nCmdShow);
	UpdateWindow(hwndMain);

	return TRUE;
}



/*
  Globals: pszAppName

  Notes:  generic
 */
BOOL InitApplication ( HINSTANCE hInstance )
{
  // Load the command bands, command bars, rebars, etc.
    INITCOMMONCONTROLSEX icex;
    icex.dwSize = sizeof(icex);
    icex.dwICC = ICC_BAR_CLASSES | ICC_COOL_CLASSES;
    InitCommonControlsEx(&icex);
	
	WNDCLASS wc;
	BOOL f;

	wc.style = CS_HREDRAW | CS_VREDRAW ;
	wc.lpfnWndProc = (WNDPROC)WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hIcon = NULL;
	wc.hInstance = hInstance;
	wc.hCursor = NULL;
	wc.hbrBackground = (HBRUSH) GetStockObject( WHITE_BRUSH );
	wc.lpszMenuName = NULL;
	wc.lpszClassName = pszAppName;
	
	f = (RegisterClass(&wc));


	return f;
}



/* Globals: pszAppName, pszTitle.

   Notes:  generic
 */
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG msg;
	HWND hPingWnd = FindWindow(pszAppName, pszTitle);

	if (hPingWnd) {
	  SetForegroundWindow(hPingWnd);
	  return 0;
	}

	if ( !hPrevInstance )
	{
		if ( !InitApplication ( hInstance ) )
		{
			return (FALSE); 
		}

	}
	if ( !InitInstance( hInstance, nCmdShow )  )
	{
		return (FALSE);
	}

	
    while (GetMessage(&msg, NULL, 0, 0) == TRUE)
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg); 
	}
	return (msg.wParam);
}
